/* ========================================================================
   =================         TVicHW32  DLL             ====================
   =================            LPT stuff              ====================
   ==========            Shareware Version 4.0                  ===========
   ==========     Copyright (c) 1999 Victor I.Ishikeev          ===========
   ========================================================================
   ==========         mail to "tools@entechtaiwan.com"          ===========
   ======================================================================== */

#include <windows.h>
#include <winioctl.h>
#include <stdlib.h>
#include <stdio.h>

#include "TVicVar.h"
#include "common.h"
#include "TVicHW32.h"


const BOOL PinsPort[] = {2,0,0,0,0,0,0,0,0,1,1,1,1,2,1,2,2};
const BOOL Negative[] = 

	{TRUE,                             // -STROBE, pin 1,     Base+2,
	 FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE, FALSE,//  DATA,    pins 2..9, Base+0
     FALSE,                            //  ACKWL,  pin 10,    Base+1
     TRUE,                             //  BUSY,   pin 11,    Base+1
     FALSE,                            //  PE,     pin 12,    Base+1
     FALSE,                            //  SLCT,   pin 13,    Base+1
     TRUE,                             // -AUTOFD, pin 14,    Base+2
     FALSE,                            // -ERROR,  pin 15,    Base+1
     FALSE,                            //  INIT,   pin 16,    Base+2
     TRUE};                            // -SLCTIN  pin 17,    Base+2

const BYTE MaskPins[] =
	{0x01,                             // -STROBE,  pin 1,     Base+2,
     0x01,							   //  DATA0,   pin 2,     Base+0
	 0x02,							   //  DATA1,   pin 3,     Base+0
	 0x04,							   //  DATA2,   pin 4,     Base+0
	 0x08,						       //  DATA3,   pin 5,     Base+0
	 0x10,							   //  DATA4,   pin 6,     Base+0
	 0x20,							   //  DATA5,   pin 7,     Base+0
	 0x40,							   //  DATA6,   pin 8,     Base+0
	 0x80,							   //  DATA7,   pin 9,     Base+0
     0x40,                             // -ACKWL,   pin 10,    Base+1
     0x80,                             //  BUSY,    pin 11,    Base+1
     0x20,                             //  PE,      pin 12,    Base+1
     0x10,                             //  SLCT,    pin 13,    Base+1
     0x02,                             // -AUTOFD,  pin 14,    Base+2
     0x08,                             // -ERROR,   pin 15,    Base+1
     0x04,                             //  INIT,    pin 16,    Base+2
     0x08};                            // -SLCTIN,  pin 17,    Base+2


UCHAR VICFN AddNewLPT(PVicHW32Descriptor HW32, USHORT PortBaseAddress)
{
	UCHAR n = 1;

	if (HW32 && HW32->fOpenDrive)  {

		for (n=0; n<HW32->fLPTs; n++)
			if (HW32->fLPTAddresses[n] == PortBaseAddress) return n+1;

        n = HW32->fLPTs;
		++HW32->fLPTs;

		HW32->fLPTAddresses[n] = PortBaseAddress;

		++n;

		SetLPTNumber(HW32,n);

	}

	return n;

}




UCHAR VICFN GetLPTNumber(PVicHW32Descriptor HW32) 
{
	if (HW32 && HW32->fOpenDrive)
		return (HW32->fLPTNumber);
	else
		return 1;
}

void  VICFN SetLPTNumber(PVicHW32Descriptor HW32,UCHAR nNewValue) 
{
	
	if (HW32 && HW32->fOpenDrive && (nNewValue<=HW32->fLPTs)) {
		HW32->fLPTBasePort = HW32->fLPTAddresses[nNewValue-1];
		HW32->fDataReadPorts[0]  = GetPortByte(HW32, HW32->fLPTBasePort);
		HW32->fDataReadPorts[1]  = GetPortByte(HW32, HW32->fLPTBasePort+1);
		HW32->fDataReadPorts[2]  = GetPortByte(HW32, HW32->fLPTBasePort+2);
		HW32->fDataWritePorts[0] = GetPortByte(HW32, HW32->fLPTBasePort);
		HW32->fDataWritePorts[1] = GetPortByte(HW32, HW32->fLPTBasePort+1);
		HW32->fDataWritePorts[2] = GetPortByte(HW32, HW32->fLPTBasePort+2);
        HW32->LptIrqForced = (HW32->fDataWritePorts[2] & 0x10);
		HW32->fLPTNumber = (BYTE)nNewValue;
	};

	
}

UCHAR  VICFN GetLPTNumPorts(PVicHW32Descriptor HW32) 
{
	if (HW32 && HW32->fOpenDrive)
	    return HW32->fLPTs;
	else return 0;
}

ULONG VICFN GetLPTBasePort(PVicHW32Descriptor HW32) 
{
    if (HW32 && HW32->fOpenDrive)
	    return HW32->fLPTBasePort;
	else return 0;
}


BOOL VICFN GetPin(PVicHW32Descriptor HW32,UCHAR nPin) 
{
    UCHAR data;
	USHORT ofs,n;
	BOOL Result = FALSE;

    if (!HW32 || !HW32->fOpenDrive || (nPin>17) || (nPin<=0)) return Result;
	n = nPin - 1;
	ofs = PinsPort[n];
	data = GetPortByte(HW32,(USHORT)(HW32->fLPTBasePort+ofs));
    Result = (data & MaskPins[n]) != 0;
	if (Negative[n]) Result = !Result;
	if (Result) Result = TRUE;
	return Result;	
}

void VICFN SetPin(PVicHW32Descriptor HW32,UCHAR nPin, BOOL bNewValue) 
{
   BYTE   data;
   USHORT ofs,n;
   if (!HW32 || !HW32->fOpenDrive || (nPin>17) || (nPin<=0)) return;

   if (bNewValue) bNewValue = TRUE;

   n = nPin-1;
   ofs    = PinsPort[n];
   data   = HW32->fDataWritePorts[ofs];
   if (bNewValue != Negative[n]) data = data | MaskPins[n];
   else data = data & (~MaskPins[n]);
   SetPortByte(HW32,(short)(HW32->fLPTBasePort+ofs),data);
   HW32->fDataWritePorts[ofs] = data;
}


void VICFN LPTStrobe(PVicHW32Descriptor HW32) 
{
	int i;
    if (!HW32 || !HW32->fOpenDrive) return;
    SetPin(HW32,1,FALSE);
    for (i = 0; i<1000; i++);
    SetPin(HW32,1,TRUE);
}

BOOL VICFN GetLPTAckwl(PVicHW32Descriptor HW32) 
{
	return (!GetPin(HW32,10));
}

BOOL VICFN GetLPTBusy(PVicHW32Descriptor HW32) 
{
	return (GetPin(HW32,11));
}

BOOL VICFN GetLPTPaperEnd(PVicHW32Descriptor HW32) 
{
    return (GetPin(HW32,12));
}

BOOL VICFN GetLPTSlct(PVicHW32Descriptor HW32) 
{
	return (!GetPin(HW32,13));
}

void VICFN LPTAutofd(PVicHW32Descriptor HW32,BOOL Flag) 
{
	SetPin(HW32,14,!Flag);
}

BOOL VICFN GetLPTError(PVicHW32Descriptor HW32) 
{
	return (!GetPin(HW32,15));
}

void VICFN LPTInit(PVicHW32Descriptor HW32)
{
	int i;
    SetPin(HW32,16,FALSE);
	for (i=1; i<1000; i++);
	SetPin(HW32,16,TRUE);

}

void VICFN LPTSlctIn(PVicHW32Descriptor HW32) 
{
	UCHAR data;
	if (!HW32 || !HW32->fOpenDrive) return;
    SetPin(HW32,16,FALSE);
	data = HW32->fDataWritePorts[2] & 0xf7;
    HW32->fDataWritePorts[2] = (BYTE)data;
	SetPortByte(HW32,(short)(HW32->fLPTBasePort+2),data);
}

BOOL VICFN LPTPrintChar(PVicHW32Descriptor HW32,UCHAR ch) 
{
	BOOL Result = FALSE;
	if (!HW32 || !HW32->fOpenDrive) return Result;
	if (HW32->fOpenDrive) {
		SetPortByte(HW32,HW32->fLPTBasePort,ch);
		if (!GetLPTError(HW32) && !GetLPTBusy(HW32)) {
			LPTStrobe(HW32);
			Result = TRUE;
        };
	};
	return Result;
}

void  VICFN ForceIrqLPT(PVicHW32Descriptor HW32, BOOL IrqEnable)
{
	UCHAR data;

	if (HW32 && HW32->fOpenDrive) {

		HW32->LptIrqForced = IrqEnable;

		if (IrqEnable) {

			data = HW32->fDataWritePorts[2] | 0x10;

		}
		else {

 		     data = HW32->fDataWritePorts[2] & 0xEF;

		}

		SetPortByte(HW32,(short)(HW32->fLPTBasePort+2),data);

	}
}

void VICFN SetLPTReadMode(PVicHW32Descriptor HW32) 
{
    if (!HW32 || !HW32->fOpenDrive) return;

	HW32->fDataWritePorts[2]  = HW32->fDataWritePorts[2] | 0x20;

    SetPortByte(HW32,(ULONG)(HW32->fLPTBasePort+2),(UCHAR)(HW32->fDataWritePorts[2]));
}


void VICFN SetLPTWriteMode(PVicHW32Descriptor HW32) 
{
    if (!HW32 || !HW32->fOpenDrive) return;

	HW32->fDataWritePorts[2]  = HW32->fDataWritePorts[2] & 0xDF;
    SetPortByte(HW32,(ULONG)(HW32->fLPTBasePort+2),(UCHAR)(HW32->fDataWritePorts[2]));
}


USHORT VICFN AcquireLPT    ( PVicHW32Descriptor HW32, USHORT LPTNumber)
{
	ULONG nByte;
	BOOL  q;
	
	char   LPTName[5];

    if (!HW32 || !HW32->fOpenDrive) 
	    return LPT_ACQUIRE_NOT_OPENED ; // Driver not opened

	if ((LPTNumber>3) || (LPTNumber<=0) )
	    return LPT_ACQUIRE_BAD_PORT;

    if (HW32->fLptAcquired[LPTNumber-1] == LPT_ACQUIRE_SUCCESS ) 
	    return LPT_ACQUIRE_SUCCESS;

    HW32->fLptAcquired[LPTNumber-1] = LPT_NOT_ACQUIRED;

	
    switch (LPTNumber) {

	    case 1: strcpy(LPTName,"LPT1"); break;
		case 2: strcpy(LPTName,"LPT2"); break;
		case 3: strcpy(LPTName,"LPT3"); break;
		default:strcpy(LPTName,"LPT1"); break;

	}

	if (HW32->fWin95) {

		q = DeviceIoControl(HW32->hDrv,
			                CtlCode(ACQUIRE_LPT),
			                LPTName,4,
							NULL,0,
							&nByte,NULL); 
		if (q) {

           HW32->fLptAcquired[LPTNumber-1] = LPT_ACQUIRE_SUCCESS;
	       return LPT_ACQUIRE_SUCCESS;

		}


	}
	else {


		HW32->hLptAcquired[LPTNumber-1] = 
			CreateFile(LPTName, 
		               GENERIC_WRITE | GENERIC_READ, 
				       0, 
					   NULL, 
					   OPEN_EXISTING,
		               FILE_FLAG_OVERLAPPED, 
					   0);


        if (HW32->hLptAcquired[LPTNumber-1] != INVALID_HANDLE_VALUE) {

        HW32->fLptAcquired[LPTNumber-1] = LPT_ACQUIRE_SUCCESS;
	    return LPT_ACQUIRE_SUCCESS;

		}
	}

    return LPT_ACQUIRE_REFUSED;

 }

void   VICFN ReleaseLPT    ( PVicHW32Descriptor HW32, USHORT LPTNumber)
{
    ULONG nByte;

    if (!HW32 || !HW32->fOpenDrive) return; // Driver not opened

    if ((LPTNumber>3) || (LPTNumber<=0) ) return;
	    
    if (HW32->fLptAcquired[LPTNumber-1] == LPT_ACQUIRE_SUCCESS ) {

		if (HW32->fWin95) {

            switch (LPTNumber) {

		    case 1: DeviceIoControl(HW32->hDrv,CtlCode(RELEASE_LPT),"LPT1",4,NULL,0,&nByte,NULL); break;
	        case 2: DeviceIoControl(HW32->hDrv,CtlCode(RELEASE_LPT),"LPT2",4,NULL,0,&nByte,NULL); break;
	        case 3: DeviceIoControl(HW32->hDrv,CtlCode(RELEASE_LPT),"LPT3",4,NULL,0,&nByte,NULL); break;
			default:DeviceIoControl(HW32->hDrv,CtlCode(RELEASE_LPT),"LPT1",4,NULL,0,&nByte,NULL); break;

			}		
		}

		else

	      CloseHandle(HW32->hLptAcquired[LPTNumber-1]);

	}

    HW32->fLptAcquired[LPTNumber-1] = LPT_NOT_ACQUIRED;

}

USHORT VICFN IsLPTAcquired ( PVicHW32Descriptor HW32, USHORT LPTNumber)
{
	USHORT Rez = LPT_NOT_ACQUIRED;

	if (!HW32 || !HW32->fOpenDrive) 
	    return LPT_ACQUIRE_NOT_OPENED ; // Driver not opened

    if ((LPTNumber>3) || (LPTNumber<=0) )
	    return LPT_ACQUIRE_BAD_PORT;

		
	if (HW32->fLptAcquired[LPTNumber-1] == LPT_ACQUIRE_SUCCESS) 
		return LPT_ACQUIRE_SUCCESS;
	 
    return Rez;
}
