/* ========================================================================
   =================    TVicHW32  DLL interface        ====================
   =================         Port I/O stuff            ====================
   ==========            Shareware Version 5.0                  ===========
   ==========     Copyright (c) 2000 Victor I.Ishikeev          ===========
   ========================================================================
   ==========         mail to "tools@entechtaiwan.com"          ===========
   ======================================================================== */

#include <windows.h>
#include <winioctl.h>
#include <stdlib.h>


#include "TVicVar.h"
#include "common.h"
#include "TVicHW32.h"


extern ULONG CtlCode(ULONG Code);
extern BOOL IsWow64;


void ModifyLptWritePorts(PVicHW32Descriptor HW32, ULONG PortAddr,UCHAR DataPort)
{
	int n = PortAddr - HW32->fLPTBasePort;
	if ( (n>=0) && (n<=2) )	{
		HW32->fDataWritePorts[n] = DataPort;
		if (n == 2) HW32->LptIrqForced = (DataPort & 0x10);
	}
}

void ModifyLptReadPorts(PVicHW32Descriptor HW32, ULONG PortAddr,UCHAR DataPort)
{
	int n = PortAddr - HW32->fLPTBasePort;
	if ( (n>=0) && (n<=2) )	HW32->fDataReadPorts[n] = DataPort;
}

BOOL VICFN GetHardAccess(PVicHW32Descriptor HW32) 
{
	if (HW32 && HW32->fOpenDrive)
		return HW32->fHardAccess;
	else
    	return TRUE;
}

void  VICFN SetHardAccess(PVicHW32Descriptor HW32, BOOL bNewValue) 
{
	if (HW32 && HW32->fOpenDrive)
	{
		HW32->fHardAccess = TRUE;

		if (IsWow64 || bNewValue) 
			return;
        else
		{
#ifndef NO_RING0_PORT
			if (!HW32->fCallGateCreated)
			{
				HW32->fCallGateCreated = PrepareRing0Call(HW32);
				if (!HW32->fCallGateCreated) return;
			}
#else
			HW32->fCallGateCreated = FALSE;
#endif
#ifndef HARD_ACCESS_ONLY
			HW32->fHardAccess = bNewValue;
#else
			HW32->fHardAccess = TRUE;
#endif
		}
	}

}

UCHAR  VICFN GetPortByte(PVicHW32Descriptor HW32, ULONG PortAddr) 
{
	DWORD nByte, PortNumber, DataPort;

	WORD   farcall[3];
	
	if (!HW32 || !HW32->fOpenDrive) return 0xff;

	if (HW32->fHardAccess || IsWow64) {

		PortNumber= (DWORD)PortAddr;
		DeviceIoControl(HW32->hDrv,
						CtlCode(READ_PORT),
						&PortAddr, sizeof(PortNumber),
						&DataPort, sizeof(DataPort),
						&nByte, NULL);
	} 
	else {

		farcall[0] = 0; 
		farcall[1] = 0; 
		farcall[2]= (WORD) HW32->CallGateInfo.CallGateSelector;

		__asm {
			mov  edx,PortAddr
            mov  edi,1
            call fword ptr [farcall]
			mov  DataPort,eax
		}
	}

	ModifyLptReadPorts(HW32,PortAddr,(UCHAR)(LOWORD(DataPort) & 0x00ff));
	return (UCHAR)(LOWORD(DataPort) & 0x00ff);

}


USHORT  VICFN GetPortWord(PVicHW32Descriptor HW32, ULONG PortAddr) 
{
	DWORD nByte, PortNumber, DataPort;

	WORD   farcall[3];
	
	if (!HW32 || !HW32->fOpenDrive) return 0xff;

	if (HW32->fHardAccess || IsWow64) {

		PortNumber= (DWORD)PortAddr;
		DeviceIoControl(HW32->hDrv,
						CtlCode(READ_PORTW),
						&PortAddr, sizeof(PortNumber),
						&DataPort, sizeof(DataPort),
						&nByte, NULL);
	} 
	else {

		farcall[0] = 0; 
		farcall[1] = 0; 
		farcall[2]= (WORD) HW32->CallGateInfo.CallGateSelector;

		__asm {
			mov  edx,PortAddr
            mov  edi,2
            call fword ptr [farcall]
			mov  DataPort,eax
		}

	}

	ModifyLptReadPorts(HW32,PortAddr,(UCHAR)DataPort);

	return LOWORD(DataPort);

}

ULONG VICFN GetPortLong(PVicHW32Descriptor HW32,ULONG PortAddr) 
{
    DWORD nByte, PortNumber, DataPort;

	WORD   farcall[3];
	
	if (!HW32 || !HW32->fOpenDrive) return 0xff;

	if (HW32->fHardAccess || IsWow64) {

		PortNumber= (DWORD)PortAddr;
		DeviceIoControl(HW32->hDrv,
						CtlCode(READ_PORTL),
						&PortAddr, sizeof(PortNumber),
						&DataPort, sizeof(DataPort),
						&nByte, NULL);
	} 
	else {

		nByte = (DWORD)&DataPort;

		farcall[0] = 0; 
		farcall[1] = 0; 
		farcall[2]= (WORD) HW32->CallGateInfo.CallGateSelector;

		__asm {
			mov  edx,PortAddr
            mov  edi,3
            call fword ptr [farcall]
			mov  DataPort,eax
		}
	}
	
    ModifyLptReadPorts(HW32,PortAddr,(UCHAR)DataPort);
	return DataPort;	

}


void  VICFN SetPortByte(PVicHW32Descriptor HW32, ULONG PortAddr, UCHAR nNewValue) 
{
	DWORD nByte;
	struct _Rec {
		DWORD PortNumber;
		DWORD DataPort;
	} Rec;

	WORD   farcall[3];

	if (!HW32) return;

	ModifyLptWritePorts(HW32,PortAddr,(UCHAR)nNewValue);

	if (!HW32->fOpenDrive)
		return;


	if (HW32->fHardAccess || IsWow64) {
		Rec.PortNumber	= (DWORD)PortAddr;
		Rec.DataPort	= (DWORD)nNewValue & 0x000000ff;
		DeviceIoControl(HW32->hDrv,
						CtlCode(WRITE_PORT),
						&Rec, sizeof(Rec), NULL, 0,
						&nByte, NULL);
	} 
	else {
		
        farcall[0] = 0;
	    farcall[1] = 0;
	    farcall[2]= (WORD) HW32->CallGateInfo.CallGateSelector;

	    __asm {
             mov edx,PortAddr
             mov edi,4
             mov al,nNewValue
             call fword ptr [farcall]
		}
    }
}

void  VICFN SetPortWord(PVicHW32Descriptor HW32, ULONG PortAddr, USHORT nNewValue) 
{
	DWORD nByte = 0;
	struct _Rec {
		DWORD PortNumber;
		DWORD DataPort;
	} Rec;

	WORD   farcall[3];

	if (!HW32) return;

	ModifyLptWritePorts(HW32,PortAddr,(UCHAR)nNewValue);

	if (!HW32->fOpenDrive)
		return;


	if (HW32->fHardAccess || IsWow64) {
		Rec.PortNumber	= (DWORD)PortAddr;
		Rec.DataPort	= (DWORD)nNewValue & 0x0000ffff;
		DeviceIoControl(HW32->hDrv,
						CtlCode(WRITE_PORTW),
						&Rec, sizeof(Rec), NULL, 0,
						&nByte, NULL);
	} 
	else {
		
        farcall[0] = 0;
	    farcall[1] = 0;
	    farcall[2]= (WORD) HW32->CallGateInfo.CallGateSelector;

		 __asm {
             mov  edx,PortAddr
             mov  edi,5
             mov  ax,nNewValue
             call fword ptr [farcall]
		}
	
    }
}


void VICFN SetPortLong(PVicHW32Descriptor HW32,ULONG PortAddr, ULONG nNewValue) 
{
	DWORD nByte = 0;
	struct _Rec {
		DWORD PortNumber;
		DWORD DataPort;
	} Rec;

    WORD   farcall[3];

	if (!HW32) return;

	ModifyLptWritePorts(HW32,PortAddr,(UCHAR)nNewValue);

	if (!HW32->fOpenDrive)
		return;


	if (HW32->fHardAccess || IsWow64) {
		Rec.PortNumber	= (DWORD)PortAddr;
		Rec.DataPort	= (DWORD)nNewValue;
		DeviceIoControl(HW32->hDrv,
						CtlCode(WRITE_PORTL),
						&Rec, sizeof(Rec), NULL, 0,
						&nByte, NULL);
	} 
	else {
		
        farcall[0] = 0;
	    farcall[1] = 0;
	    farcall[2]= (WORD) HW32->CallGateInfo.CallGateSelector;

		__asm {
             mov  edx,PortAddr
             mov  edi,6
             mov  eax,nNewValue
             call fword ptr [farcall]
		}
	
    }
	
	
}

void  VICFN ReadPortFIFO ( PVicHW32Descriptor HW32,
						   PFIFO_RECORD       pPortRec)
                           
{
    ULONG BufSize,nByte;

	if (!HW32 || !(HW32->fOpenDrive)) return;


    BufSize = 8+pPortRec->NumPorts;	
    
    DeviceIoControl(HW32->hDrv,
                    CtlCode(READ_PORT_FIFO),
                    pPortRec,BufSize,
                    pPortRec,BufSize,
                    &nByte,NULL);
}    

void  VICFN ReadPortWFIFO ( PVicHW32Descriptor HW32,
					        PFIFO_RECORDW      pPortRec)
                            
{
    ULONG BufSize,nByte;
	
	if (!HW32 || !(HW32->fOpenDrive)) return;

    BufSize = 8+2*(pPortRec->NumPorts);	
	   
    DeviceIoControl(HW32->hDrv,
                    CtlCode(READ_PORT_FIFOW),
                    pPortRec,BufSize,
                    pPortRec,BufSize,
                    &nByte,NULL);

}    

void  VICFN ReadPortLFIFO( PVicHW32Descriptor HW32,
			               PFIFO_RECORDL      pPortRec)
                           
{
    ULONG BufSize,nByte;

	if (!HW32 || !(HW32->fOpenDrive)) return;

	BufSize = 8+4*(pPortRec->NumPorts);
		
    DeviceIoControl(HW32->hDrv,
                    CtlCode(READ_PORT_FIFOL),
                    pPortRec,BufSize,
                    pPortRec,BufSize,
                    &nByte,NULL);

}    


void  VICFN WritePortFIFO( PVicHW32Descriptor HW32,
			               PFIFO_RECORD       pPortRec)
    
{
    ULONG BufSize,nByte;

	if (!HW32 || !(HW32->fOpenDrive)) return;

    BufSize = 8+pPortRec->NumPorts;
    
    DeviceIoControl(HW32->hDrv,
                    CtlCode(WRITE_PORT_FIFO),
                    pPortRec,BufSize,
					pPortRec,BufSize,
                    &nByte,NULL);
    
}    

void  VICFN WritePortWFIFO( PVicHW32Descriptor HW32,
			                PFIFO_RECORDW      pPortRec)
    
{
    ULONG BufSize,nByte;

	if (!HW32 || !(HW32->fOpenDrive)) return;

    BufSize = 8+2*(pPortRec->NumPorts);
    
    DeviceIoControl(HW32->hDrv,
                    CtlCode(WRITE_PORT_FIFOW),
                    pPortRec,BufSize,
					pPortRec,BufSize,
                    &nByte,NULL);
}    

void  VICFN WritePortLFIFO( PVicHW32Descriptor HW32,
			                PFIFO_RECORDL      pPortRec)
    
{
	ULONG BufSize,nByte;

	if (!HW32 || !(HW32->fOpenDrive)) return;

	BufSize = 8+4*(pPortRec->NumPorts);
    
    DeviceIoControl(HW32->hDrv,
                    CtlCode(WRITE_PORT_FIFOL),
                    pPortRec,BufSize,
					pPortRec,BufSize,
                    &nByte,NULL);
}